/*	bb.h

	Basic Block compiler in C:  Header File

	2017 by H. Dietz
*/

#include	<stdlib.h>
#include	<stdio.h>
#include	<ctype.h>
#include	<string.h>

#define	DEBUG			/* cause debugging printfs */
#undef	DEBUG

typedef	int	konst;		/* type of a constant value */
typedef	int	label;		/* type of a label value */
typedef	int	opcode;		/* type of an opcode */

/*	Symbol table (variable) struct... */
#define	var	struct _var
var {
	char	*text;		/* pointer to string */
	int	type;		/* token type of variable (e.g., IF) */
	int	dim;		/* size of dimension (1 means scalar) */
	int	deflev;		/* scope level of definition */
	int	defblk;		/* block number of definition */
};

/*	Tuple struct... */
#define	tuple	struct _tuple
tuple {
	tuple	*next, *prev;	/* sequential order links */
	opcode	oarg;		/* tuple opcode */
	tuple	*targ[2];	/* tuple arguments */
	konst	carg;		/* constant argument */
	label	larg[2];	/* label arguments */
	var	*varg;		/* variable name argument */
	int	refs;		/* how many refs to this? */
};

/*	token types... */
#define	ADD	'+'
#define	SUB	'-'
#define	AND	'&'
#define	OR	'|'
#define	XOR	'^'
#define	GT	'>'
#define	GE	512
#define	EQ	513
#define	SSL	514
#define	SSR	515
#define	CONST	516
#define	LD	517
#define	LDX	518
#define	ST	519
#define	STX	520
#define	LAB	521
#define	SEL	522
#define	KILL	523

#define	WORD	524
#define	NE	525
#define	LT	'<'
#define	LE	526

#define	INT	1024
#define	IF	1025
#define	ELSE	1026
#define	WHILE	1027
#define	BREAK	1028
#define	CONT	1029
#define	UNDEF	2048

/*	bb2.C */
extern	void	prog(void);		/* parser entry point */

/*	bb3.C */
extern	int	lookt;		/* next token of input */
extern	var	*lookv;		/* next variable */
extern	konst	lookk;		/* next constant */
extern	char	looks[513];	/* next string */
extern	char	errbuf[1025];	/* error message buffer */
extern	int nextt(void);	/* get next token from input */
extern	void error(char *fmat);
extern	int need(int x, char *y);

/*	bb4.C */
extern	int newscope(void);	/* make a new scope */
extern	int endscope(void);	/* end the current scope */
extern	var *findv(register char *s);		/* find var entry in symbol table */
extern	var *makev(register char *s);		/* make var entry in symbol table */
extern	void initv(void);	/* initialize symbol table */

/*	bb5.C */
extern	tuple	*binop(opcode o, tuple *t1, tuple *t2);
extern	tuple	*cop(konst c);
extern	tuple	*ldop(var *v);
extern	tuple	*ldxop(var *v, tuple *t);
extern	tuple	*stop(var *v, tuple *t);
extern	tuple	*stxop(var *v, tuple *t1, tuple *t2);
extern	tuple	*labop(label l);
extern	tuple	*selop(tuple *t, label l1, label l2);
extern	tuple	*killop(var *v);
extern	void	codegen(void);
